# -*- coding: utf-8 -*-

import re
import os
import sys

local = {}
lfn = os.path.join(os.path.split(__file__)[0], 'local.py')
if os.path.exists(lfn):
    execfile(lfn, local, local)

OPTIONS = [('equation', 'mhd'),
           ('model', 'orszagtang'),
           ('multi', 'single'),
           ('rng', 'well512a'),
           ('solver', 'hll'),
           ('space', 'o1'),
           ('topography','none')
          ]

META_RE = re.compile(r'///::(\w+)::\s+(.+)$')

BC_FUNC_VALUES = ['N', 'l', 'N', 'l', 'N', 'l']
BC_STD_VALUES = ['i', 'j', 'k', 'i', 'j', 'k']
BC_FUNC = """
void set_ghost_%s()
{
  for (int i = 0; i < %sX%s; i++) {
    for (int j = 0; j < %sY%s; j++) {
      for (int k = 0; k < %sZ%s; k++) {
        %s;
        calc_xtra(Xtra[%s][%s][%s], U[%s][%s][%s], %s, %s, %s);
        %s;
        calc_xtra(Xtra[%s][%s][%s], U[%s][%s][%s], %s, %s, %s);
      }
    }
  }
}
"""
BC_FUNC_SW = """
void set_ghost_%s()
{
  for (int i = 0; i < %sX%s; i++) {
    for (int j = 0; j < %sY%s; j++) {
      for (int k = 0; k < %sZ%s; k++) {
        %s;
        %s;
        calc_xtra(Xtra[%s][%s][%s], U[%s][%s][%s], %s, %s, %s);
        %s;
        %s;
        calc_xtra(Xtra[%s][%s][%s], U[%s][%s][%s], %s, %s, %s);
      }
    }
  }
}
"""
BC_FUNC_PARALLEL = """
void set_ghost_%s()
{
  for (int i = 0; i < %sX%s; i++) {
    for (int j = 0; j < %sY%s; j++) {
      for (int k = 0; k < %sZ%s; k++) {
        if (mpibc[%s])
          U[%s][%s][%s] = mpi_rbuf[mpi_bc_rbuf[%s]][(i*%sY%s + j)*%sZ%s + k];
        else %s;
        calc_xtra(Xtra[%s][%s][%s], U[%s][%s][%s], %s, %s, %s);
        if (mpibc[%s])
          U[%s][%s][%s] = mpi_rbuf[mpi_bc_rbuf[%s]][(i*%sY%s + j)*%sZ%s + k];
        else %s;
        calc_xtra(Xtra[%s][%s][%s], U[%s][%s][%s], %s, %s, %s);
      }
    }
  }
}
"""
BC_FUNC_PARALLEL_SW = """
void set_ghost_%s()
{
  for (int i = 0; i < %sX%s; i++) {
    for (int j = 0; j < %sY%s; j++) {
      for (int k = 0; k < %sZ%s; k++) {
        if (mpibc[%s]) {
          U[%s][%s][%s] = mpi_rbuf[mpi_bc_rbuf[%s]][(i*%sY%s + j)*%sZ%s + k];
        } else {
          %s;
          %s;
        }
        calc_xtra(Xtra[%s][%s][%s], U[%s][%s][%s], %s, %s, %s);
        if (mpibc[%s]) {
          U[%s][%s][%s] = mpi_rbuf[mpi_bc_rbuf[%s]][(i*%sY%s + j)*%sZ%s + k];
        } else {
          %s;
          %s;
        }
        calc_xtra(Xtra[%s][%s][%s], U[%s][%s][%s], %s, %s, %s);
      }
    }
  }
}
"""
BC_STD = "U[%s][%s][%s] = U[%s][%s][%s]"
BC_CALL = "bc_%s_%s%s(U[%s][%s][%s],i,j,k)"
BC_CALL_DEF = "void bc_%s_%s%s(ConservedVars &U, int i, int j, int k);\n"
BC_INIT_CALL = "bc_%s_%s%s_init();\n"

def bc_single (f, spec):
    f.write('#include "main.h"\n')
    init = []
    for n in range(6):
        if spec[n] == 'REPEAT': spec[n] = 1
        elif spec[n] == 'NEUMANN': spec[n] = 2
        else:
            if spec[n][-1] == '*':
                spec[n] = spec[n][:-1]
                d = (spec[n].lower(), "xyzxyz"[n], "llluuu"[n])
                init.append('  '+BC_INIT_CALL%d)
                f.write('void '+BC_INIT_CALL%d)
            else:
                d = (spec[n].lower(), "xyzxyz"[n], "llluuu"[n])
            f.write(BC_CALL_DEF%d)
    for b in range(3):
        fv = ['xyz'[b]] + BC_FUNC_VALUES
        fv[1+2*b]   = 'G'
        fv[1+2*b+1] = ''
        ls = BC_STD
        lv = BC_STD_VALUES+[]
        us = BC_STD
        uv = BC_STD_VALUES+[]
        lv[b] = '-1-'+lv[b]
        uv[b] += '+N'+'XYZ'[b] + 'l'
        lsrc = lv[:3]*3
        usrc = uv[:3]*3
        if spec[b] == 1:
            lv[3+b] = 'N'+'XYZ'[b]+'l'+'-1-'+lv[3+b]
        elif spec[b] != 2:
            ls = BC_CALL
            lv = [spec[b].lower(), 'xyz'[b], 'l']+lv[:3]
        if spec[b+3] == 2:
            uv[3+b] = 'N'+'XYZ'[b]+'l'+'-1-'+uv[3+b]
        elif spec[b+3] != 1:
            us = BC_CALL
            uv = [spec[b+3].lower(), 'xyz'[b], 'u']+uv[:3]
        fv += [ls%tuple(lv)]+lsrc+[us%tuple(uv)]+usrc
        f.write(BC_FUNC % tuple(fv))
    f.write("\nvoid bc_init()\n{\n"+''.join(init)+"}\n")
    f.close()

#SW_STD = "U[%s][%s][%s].h -= b_cell_center(%s,%s) - b_cell_center(%s,%s)"
SW_STD = "U[%s][%s][%s].h -= Xtra[%s][%s][0].b - Xtra[%s][%s][0].b"

def bc_single_sw (f, spec):
    f.write('#include "main.h"\n')
    init = []
    for n in range(6):
        if spec[n] == 'REPEAT': spec[n] = 1
        elif spec[n] == 'NEUMANN': spec[n] = 2
        else:
            if spec[n][-1] == '*':
                spec[n] = spec[n][:-1]
                d = (spec[n].lower(), "xyzxyz"[n], "llluuu"[n])
                init.append('  '+BC_INIT_CALL%d)
                f.write('void '+BC_INIT_CALL%d)
            else:
                d = (spec[n].lower(), "xyzxyz"[n], "llluuu"[n])
            f.write(BC_CALL_DEF%d)
    for b in range(3):
        fv = ['xyz'[b]] + BC_FUNC_VALUES
        fv[1+2*b]   = 'G'
        fv[1+2*b+1] = ''
        ls = BC_STD
        lv = BC_STD_VALUES + []
        lv[b] = '-1-'+lv[b]
        us = BC_STD
        uv = BC_STD_VALUES + []
        uv[b] += '+N'+'XYZ'[b] + 'l'
        lsrc = lv[:3]*3
        usrc = uv[:3]*3
        if spec[b] == 1:
            lv[3+b] = 'N'+'XYZ'[b]+'l'+'-1-'+lv[3+b]
        elif spec[b] != 2:
            ls = BC_CALL
            lv = [spec[b].lower(), 'xyz'[b], 'l']+lv[:3]
        if spec[b+3] == 2:
            uv[3+b] = 'N'+'XYZ'[b]+'l'+'-1-'+uv[3+b]
        elif spec[b+3] != 1:
            us = BC_CALL
            uv = [spec[b+3].lower(), 'xyz'[b], 'u']+uv[:3]
        ls_SW = SW_STD
        lv_SW = lv[:3] + lv[0:2] + lv[3:5]
        us_SW = SW_STD
        uv_SW = uv[:3] + uv[0:2] + uv[3:5]
        fv += [ls%tuple(lv)]+[ls_SW%tuple(lv_SW)]+lsrc+[us%tuple(uv)]+[us_SW%tuple(uv_SW)]+usrc
        f.write(BC_FUNC_SW % tuple(fv))
    f.write("\nvoid bc_init()\n{\n"+''.join(init)+"}\n")
    f.close()

def bc_parallel (f, spec, settings):
    f.write('#include "main.h"\n')
    f.write('#include "multi_mpi.h"\n')
    f.write('#include "mpi.h"\n')
    init = []
    for n in range(6):
        if spec[n] == 'REPEAT':
            spec[n] = 1
            settings[1]['BC_REPEAT_'+'XYZ'[n%3]] = 1
        elif spec[n] == 'NEUMANN': spec[n] = 2
        else:
            if spec[n][-1] == '*':
                spec[n] = spec[n][:-1]
                d = (spec[n].lower(), "xyzxyz"[n], "llluuu"[n])
                init.append('  '+BC_INIT_CALL%d)
                f.write('void '+BC_INIT_CALL%d)
            else:
                d = (spec[n].lower(), "xyzxyz"[n], "llluuu"[n])
            f.write(BC_CALL_DEF%d)
    for b in range(3):
        fv = ['xyz'[b]] + BC_FUNC_VALUES
        fv[1+2*b]   = 'G'
        fv[1+2*b+1] = ''
        lm = ['XYZ'[b]] + ['i','j','k'] + ['XYZ'[b]] + ['N', 'l', 'N', 'l']
        lm[1+b] = '-1-'+lm[1+b]
        if b == 1:
          lm[5] = 'G'
          lm[6] = ''
        if b == 2:
          lm[7] = 'G'
          lm[8] = ''
        um = ['XYZ'[b]+'+NDIR'] + ['i','j','k'] + ['XYZ'[b]+'+NDIR'] + ['N', 'l', 'N', 'l']
        um[1+b] = 'N'+'XYZ'[b]+'l'+'+'+um[1+b]
        if b == 1:
          um[5] = 'G'
          um[6] = ''
        if b == 2:
          um[7] = 'G'
          um[8] = ''
        ls = BC_STD
        lv = BC_STD_VALUES+[]
        us = BC_STD
        uv = BC_STD_VALUES+[]
        lv[b] = '-1-'+lv[b]
        uv[b] += '+N'+'XYZ'[b]+'l'
        lsrc = lv[:3]*3
        usrc = uv[:3]*3
        if spec[b] == 1:
            lv[3+b] = 'N'+'XYZ'[b]+'l'+'-1-'+lv[3+b]
        elif spec[b] != 2:
            ls = BC_CALL
            lv = [spec[b].lower(), 'xyz'[b], 'l']+lv[:3]
        if spec[b+3] == 2:
            uv[3+b] = 'N'+'XYZ'[b]+'l'+'-1-'+uv[3+b]
        elif spec[b+3] != 1:
            us = BC_CALL
            uv = [spec[b+3].lower(), 'xyz'[b], 'u']+uv[:3]
        fv += lm+[ls%tuple(lv)]+lsrc+um+[us%tuple(uv)]+usrc
        f.write(BC_FUNC_PARALLEL % tuple(fv))
    f.write("\nvoid bc_init()\n{\n"+''.join(init)+"}\n")
    f.close()

def bc_parallel_sw (f, spec, settings):
    f.write('#include "main.h"\n')
    f.write('#include "multi_mpi.h"\n')
    f.write('#include "mpi.h"\n')
    init = []
    for n in range(6):
        if spec[n] == 'REPEAT':
            spec[n] = 1
            settings[1]['BC_REPEAT_'+'XYZ'[n%3]] = 1
        elif spec[n] == 'NEUMANN': spec[n] = 2
        else:
            if spec[n][-1] == '*':
                spec[n] = spec[n][:-1]
                d = (spec[n].lower(), "xyzxyz"[n], "llluuu"[n])
                init.append('  '+BC_INIT_CALL%d)
                f.write('void '+BC_INIT_CALL%d)
            else:
                d = (spec[n].lower(), "xyzxyz"[n], "llluuu"[n])
            f.write(BC_CALL_DEF%d)
    for b in range(3):
        fv = ['xyz'[b]] + BC_FUNC_VALUES
        fv[1+2*b]   = 'G'
        fv[1+2*b+1] = ''
        lm = ['XYZ'[b]] + ['i','j','k'] + ['XYZ'[b]] + ['N', 'l', 'N', 'l']
        lm[1+b] = '-1-'+lm[1+b]
        if b == 1:
          lm[5] = 'G'
          lm[6] = ''
        if b == 2:
          lm[7] = 'G'
          lm[8] = ''
        um = ['XYZ'[b]+'+NDIR'] + ['i','j','k'] + ['XYZ'[b]+'+NDIR'] + ['N', 'l', 'N', 'l']
        um[1+b] = 'N'+'XYZ'[b]+'l'+'+'+um[1+b]
        if b == 1:
          um[5] = 'G'
          um[6] = ''
        if b == 2:
          um[7] = 'G'
          um[8] = ''
        ls = BC_STD
        lv = BC_STD_VALUES+[]
        us = BC_STD
        uv = BC_STD_VALUES+[]
        lv[b] = '-1-'+lv[b]
        uv[b] += '+N'+'XYZ'[b]+'l'
        lsrc = lv[:3]*3
        usrc = uv[:3]*3
        if spec[b] == 1:
            lv[3+b] = 'N'+'XYZ'[b]+'l'+'-1-'+lv[3+b]
        elif spec[b] != 2:
            ls = BC_CALL
            lv = [spec[b].lower(), 'xyz'[b], 'l']+lv[:3]
        if spec[b+3] == 2:
            uv[3+b] = 'N'+'XYZ'[b]+'l'+'-1-'+uv[3+b]
        elif spec[b+3] != 1:
            us = BC_CALL
            uv = [spec[b+3].lower(), 'xyz'[b], 'u']+uv[:3]
        ls_SW = SW_STD
        lv_SW = lv[:3] + lv[0:2] + lv[3:5]
        us_SW = SW_STD
        uv_SW = uv[:3] + uv[0:2] + uv[3:5]
        fv += lm+[ls%tuple(lv)]+[ls_SW%tuple(lv_SW)]+lsrc+um+[us%tuple(uv)]+[us_SW%tuple(uv_SW)]+usrc
        f.write(BC_FUNC_PARALLEL_SW % tuple(fv))
    f.write("\nvoid bc_init()\n{\n"+''.join(init)+"}\n")
    f.close()

class FileMeta (dict):
    def space_list(self, k):
        r = self.get(k)
        if r is None: return []
        return re.split('\s+', r)
    
    def line_list(self, k):
        r = self.get(k)
        if r is None: return []
        return r.split('\n')

class MakeMLMC (object):
    def __init__(self, dir='.'):
        self.dir = dir
        self.opts = dict(OPTIONS)
        self.vars = {}
        self.quiet = False
        self.debug = False
        self.log = False
        self.meta_store = {}
        self.configf = 'config.py'
        self.name = ''
        self.logf = 'log.txt'
        self.norun = False
        self.parallel = False
        self.walltime = 1
        self.memory = 1024
        self.ptile = 16
        self.nodes = 1
        self.profile = False
        self.xopts = ''
    
    def read_meta(self, fn):
        fname = os.path.join(self.dir, fn)
        meta = self.meta_store.get(fname)
        if meta is not None: return meta
        meta = FileMeta()
        f = open(fname)
        for l in f:
            m = META_RE.match(l.strip())
            if m:
                o = meta.get(m.group(1))
                if o:
                    meta[m.group(1)] = o + '\n' + m.group(2)
                else:
                    meta[m.group(1)] = m.group(2)
        f.close()
        self.meta_store[fname] = meta
        return meta
    
    def get_files(self):
        files = ['mlmc_main']+[k+'_'+self.getFixedAlternative(k,v) for (k,v) in self.opts.items()]
        self.files = []
        while files:
            f = files[0]
            del files[0]
            self.files.append(f)
            files += self.depend(f+'.cpp')
    
    def get_settings(self):
        vars = self.get_vars('mlmc','main')
        consts = self.get_vars('mlmc','main', consts=True)
        (m,incs) = self.get_macros('mlmc','main')
        consts.update(m)
        for o,v in self.opts.items():
            vars.update(self.get_vars(o,v))
            consts.update(self.get_vars(o,v, consts=True))
            (m,i) = self.get_macros(o,v)
            consts.update(m)
            incs.update(i)
        vars.update(self.vars)
        if int(vars['ML']) == 1 and int(vars['L']) == 0:
          vars['COMPUTE_VARIANCE'] = '0'
        self.settings = (vars, consts, incs)
        self.parallel = (self.opts['multi'] != 'single')
        self.compiler = CompilerLocal(self.opts['multi'])
    
    def getFixedAlternative (self, c, v):
        # solver files depend on equation
        if c == 'solver':
          return v + '-' + self.opts['equation']
        else:
          return v
    
    def get_vars(self, c, v, consts=False):
        if consts: name='consts'
        else: name='vars'
        vars = self.read_meta(c+'_'+self.getFixedAlternative(c,v)+'.cpp').space_list(name)
        return dict([x.split('=') for x in vars])
    
    def get_macros(self, c, v):
        vals = {}
        incs = {}
        for m in self.read_meta(c+'_'+self.getFixedAlternative(c,v)+'.cpp').line_list('macro'):
            (n,i,v) = re.split('\s+', m, 2)
            if i == '@': i = ''
            else: i = '#include "'+i+'"\n'
            if v == '@': v = ''
            vals[n] = v
            incs[n] = i
        return (vals,incs)
    
    def depend(self, f):
        return self.read_meta(f).space_list('depends')
    
    def read_config(self, fname):
        cfg = {}
        execfile(fname, globals(), cfg)
        self.opts.update(cfg['opts'])
        self.vars.update(cfg['vars'])
    
    def set_name(self, name):
        self.configf = 'config_'+name+'.py'
        self.vars['RUNTIME_FILE'] = '"runtime_'+name+'.py"'
        self.vars['DATA_FILE'] = '"output_'+name+'.dat"'
        self.vars['TIME_FILE'] = '"times_'+name+'.py"'
        self.vars['LOG_FILE'] = '"log_'+name+'.txt"'
        self.name = '_'+name
    
    def write_config_py(self, fname):
        f = open(fname, 'w')
        f.write('opts={ '+', '.join(["'"+k+"':"+repr(v)
                                     for k,v in sorted(self.opts.items())]))
        f.write(' }\nvars={ ')
        f.write(', '.join(["'"+k+"':"+repr(v)
                           for k,v in sorted(self.vars.items())])+' }\nused={ ')
        used = {}
        used.update(self.settings[0])
        used.update(self.settings[1])
        f.write(', '.join(["'"+k+"':"+repr(v)
                          for k,v in sorted(used.items())])+' }\n')
        cluster = local.get('CLUSTER')
        if cluster:
          f.write('info={' + "'CLUSTER':'" + cluster + "' }\n")
        f.close()
    
    def show_config(self):
        if self.quiet: return
        print "Configuration Running:"
        for c,v in sorted(self.opts.items()):
            print c,':',v,'"'+self.read_meta(c+'_'+self.getFixedAlternative(c,v)+'.cpp').get('title')+'"'
        for n,v in sorted(self.settings[0].items()):
            print n,'=',v
        print
    
    def write_config_h(self):
        f = open(os.path.join(self.dir, 'config.h'), 'w')
        if self.quiet: f.write('#define QUIET 1\n')
        if self.debug: f.write('#define DEBUG %d\n'%self.debug)
        if self.log: f.write('#define LOG 1\n')
        used = {}
        used.update(self.settings[0])
        used.update(self.settings[1])
        f.write(''.join(['#define %s %s\n'%x for x in sorted(used.items())]))
        f.close()
        f = open(os.path.join(self.dir, 'include.h'), 'w')
        f.write(''.join([x for x in self.settings[2].values()]))
        f.write('#include "structures_' + self.opts['equation'] + '.h"\n')
        f.close()
    
    def bc_spec(self):
        bc = self.read_meta('model_'+self.opts['model']+'.cpp').space_list('bc')
        if len(bc) == 1:
            return bc*6
        elif len(bc) == 3:
            return bc*2
        elif len(bc) == 6:
            return bc
    
    def make_bc(self):
        spec = self.bc_spec()
        f = open(os.path.join(self.dir, 'bc.cpp'), 'w')
        if self.parallel:
            if self.opts['equation'] == 'sw':
                bc_parallel_sw (f, spec, self.settings)
            else:
                bc_parallel (f, spec, self.settings)
        else:
            if self.opts['equation'] == 'sw':
                bc_single_sw (f, spec)
            else:
                bc_single (f, spec)
    
    def default_compiler(self):
        return DEFAULT_COMPILER()
    
    def compile(self, f):
        cc = self.compiler.ccpp(f+'.cpp', f+'.o')
        if not self.quiet: print cc
        return os.system("cd "+self.dir+"; "+cc) == 0
    
    def link(self):
        lc = self.compiler.link([f+'.o' for f in self.files], 'alsvid-uq')
        if not self.quiet: print lc
        return os.system("cd "+self.dir+"; "+lc) == 0
    
    def build(self):
        self.get_settings()
        self.compiler.profile = self.profile
        self.compiler.debug = self.debug
        self.make_bc()
        self.write_config_h()
        self.show_config()
        self.get_files()
        self.write_config_py(self.configf)
        if self.log:
            f = open(logf, 'w')
            f.close()
            self.compiler.log_file = logf
        for f in self.files:
            if not self.compile(f):
                raise SystemExit
                raise Exception("Compile failed for "+f)
        if not self.link():
            raise SystemExit
            raise Exception("Link failed (see make.log)")
    
    # see mlmc_init() in the source code for explanations
    def get_multi_total (self, d):
        
        if self.parallel == True:
          MULTIX = int(d.get('MULTIX',1))
          MULTIY = int(d.get('MULTIY',1))
          MULTIZ = int(d.get('MULTIZ',1))
          MULTIM = int(d.get('MULTIM',1))
          L      = int(d.get('L',1))
          NX     = int(d.get('NX',1))
          NY     = int(d.get('NY',1))
          NZ     = int(d.get('NZ',1))
          ORDER  = int(d.get('ORDER',1))
          
          ACTIVE_DIRS = (NX > 1) + (NY > 1) + (NZ > 1)
          
          FACTOR      = 2 ** ORDER
          WORK_FACTOR = 2 ** (ACTIVE_DIRS+1 - ORDER);
          
          MULTI_TOTAL = MULTIM * MULTIX * MULTIY * MULTIZ
          
          for l in range (L-1, -1, -1):
            MULTIX_old = MULTIX
            MULTIY_old = MULTIY
            MULTIZ_old = MULTIZ
            MULTIM_old = MULTIM
            
            MULTI_ADDITIONAL_old = MULTIM * MULTIX * MULTIY * MULTIZ
            
            if MULTIX != 1: MULTIX /= 2
            if MULTIY != 1: MULTIY /= 2
            if MULTIZ != 1: MULTIZ /= 2
            
            MULTI_FACTOR = (MULTIX_old / MULTIX) * (MULTIY_old / MULTIY) * (MULTIZ_old / MULTIZ)
            
            if (MULTIM * MULTI_FACTOR) >= WORK_FACTOR:
              MULTIM = (MULTIM * MULTI_FACTOR) / WORK_FACTOR
            else:
              MULTIM = 1
            
            MULTI_ADDITIONAL = MULTIM * MULTIX * MULTIY * MULTIZ
            MULTI_TOTAL += MULTI_ADDITIONAL
            
            if WORK_FACTOR != 1 and MULTI_ADDITIONAL == 1 and MULTI_ADDITIONAL_old == 1:
              break
          
          return MULTI_TOTAL
        
        else:
          return 1
    
    def run(self):
        if self.norun: return
        d = {}
        d.update(self.settings[0])
        d.update(self.settings[1])
        
        d['nprocs']   = int(self.get_multi_total(d))
        d['walltime'] = self.walltime
        d['mem']      = self.memory
        d['ptile']    = self.ptile
        d['xopts']    = self.xopts
        d['suffix']   = ''
        
        from math import ceil
        d['nodes']    = ceil(float(d['nprocs']) / d['ptile'])
        
        if d['nprocs'] < d['ptile']:
          d['ptile'] = d['nprocs']
        
        d['name']     = 'alsvid-uq'+self.name
        
        jsc = local.get('JOB_SUBMIT')
        
        # inline run
        if jsc is None:
            run_cmd = os.path.join(self.dir, 'alsvid-uq')
            d['cmd'] = run_cmd
            if self.parallel:
                run_cmd = local.get('%s_RUN'%(self.opts['multi'].upper(),),
                                    '%(cmd)s') % d
            if os.system(run_cmd) != 0:
                raise SystemExit
            return
        
        # submission using queues
        if 'DATA_PATH' in self.vars:
          run_cmd = self.vars['DATA_PATH'][1:-1]+d['name']
        else:
          run_cmd = d['name']
        os.system( '/bin/mv %s/alsvid-uq %s' % (self.dir,run_cmd) )
        d['cmd'] = run_cmd
        
        # profiling prepocessing and executable suffix
        pp = local.get('PROFILE_PREPROC')
        if self.profile and pp:
            print pp % d
            if os.system(pp % d) != 0:
                raise SystemExit
            d['suffix'] = local.get('PROFILE_SUFFIX')
        
        if self.parallel:
            run_cmd = local.get('%s_RUN'%(self.opts['multi'].upper(),), '%(cmd)s') % d
            d['cmd'] = run_cmd
        print jsc % d
        if os.system(jsc % d) != 0:
            raise SystemExit

class Options (object):

    def __init__(self):
        self.make = MakeMLMC(os.path.join(os.path.split(__file__)[0], 'src'))

    def process_args(self):
        from optparse import OptionParser
        usage = "usage: %prog [options] [config:choice] [var=value]"
        p = OptionParser(usage)
        p.add_option('-g', '--debug', action="count",
                     help="compile debug version")
        p.add_option('-q', '--quiet', action="store_true", default=False,
                     help="run quietly (no output)")
        p.add_option('-l', '--log', action="store_true", default=False,
                     help="log output to file")
        p.add_option('-r', '--rerun', action="store_true", dest="config",
                     help="use config from config.py")
        p.add_option('-c', '--config', action="store", default=False,
                     help="specify config file")
        p.add_option('-o', '--xopts', action="store", default='',
                     help="specify extra options for job submission")
        p.add_option('-n', '--name', action="store", default='',
                     help="specify run name for output files")
        p.add_option('-x', '--norun', action="store_true", default=False,
                     help="compile only, do not run")
        p.add_option('-a', '--ask', action="count",
                     help="ask user about config options (twice: vars too)")
        p.add_option('-w', '--walltime', action="store", default='1',
                     help="specify estimated runtime limit (in hours)")
        p.add_option('-m', '--memory', action="store", default='1024',
                     help="specify the required memory (in MB, per core)")
        p.add_option('-t', '--ptile', action="store", default='16',
                     help="specify the number cores per node")
        p.add_option('-p', '--profile', action="store_true", default=False,
                     help="enable profiling")
        (opts, args) = p.parse_args()
        if opts.config == True:
            self.make.read_config('config.py')
        elif opts.config:
            self.make.read_config(opts.config)
        for a in args:
            if ':' in a:
                (c,v) = a.split(':')
                self.make.opts[c] = v
            elif '=' in a:
                (n,v) = a.split('=')
                self.make.vars[n] = v
        if opts.name:
            self.make.set_name(opts.name)
        if opts.ask: self.ask(opts.ask)
        self.make.debug = opts.debug
        self.make.quiet = opts.quiet
        self.make.log = opts.log
        self.make.norun = opts.norun
        self.make.walltime = int(opts.walltime)
        self.make.memory = int(opts.memory)
        self.make.ptile = int(opts.ptile)
        self.make.profile = int(opts.profile)
        self.make.xopts = str(opts.xopts)
    
    def get_options(self, o):
        for fn in os.listdir(self.make.dir):
            m = re.match(o+r'_([A-Za-z0-9_]+).cpp$', fn)
            if m is None: continue
            yield m.group(1), self.make.read_meta(fn).get('title')
    
    def ask(self, askn):
        print "Major Options:"
        for o,dv in OPTIONS:
            opts = dict(self.get_options(o))
            print 'Select '+o+':'
            for c,t in sorted(opts.items()):
                print c, '"'+t+'"'
            while True:
                r = raw_input(o+': ['+self.make.opts[o]+'] ')
                if r == '':
                    break
                elif r in opts:
                    self.make.opts[o] = r
                    break
                else:
                    print "Invalid value for",o
            print
        if askn == 1: return
        print "Variables:"
        self.make.get_settings()
        for vn, vv in self.make.settings[0].items():
            if vv[0] == '"' and vv[-1] == '"':
                s = 1
                vv = vv[1:-1]
            else:
                s = 0
            while True:
                nv = raw_input(vn+' ['+vv+'] = ')
                if nv == '':
                    break
                elif s:
                    self.make.vars[vn] = '"'+nv+'"'
                    break
                else:
                    try:
                        n = float(nv)
                    except ValueError:
                        print "Invalid number"
                    else:
                        self.make.vars[vn] = nv
                        break
        print

class Compiler (object):
    def __init__(self, debug=False, profile=False):
        self.log_file=None
        self.debug = debug
        self.profile = profile

class CompilerLocal (Compiler):

    DEFAULTS = {
        'compile': 'g++ -O3',
        'debug'  : 'g++ -g',
        'profile': 'g++ -g -pg',
        'libs'   : ''
        }
    
    def __init__(self, type):
        super(CompilerLocal, self).__init__()
        self.COMPILER = local.get(type.upper()+'_COMPILER')
        if self.COMPILER is None:
            self.COMPILER = local.get('COMPILER', self.DEFAULTS)
    
    def ccpp(self, s, o):
        if self.log_file: lf = ' >> '+self.log_file+' 2>&1'
        else: lf = ''
        if self.profile: opts = self.COMPILER['profile']
        elif self.debug: opts = self.COMPILER['debug']
        else: opts = self.COMPILER['compile']
        return '%s -c -o %s %s%s'%(opts, o, s, lf)
    
    def link(self, olist, x):
        if self.log_file: lf = ' >> '+self.log_file+' 2>&1'
        else: lf = ''
        if self.profile: opts = self.COMPILER['profile']
        elif self.debug: opts = self.COMPILER['debug']
        else: opts = self.COMPILER['compile']
        return '%s -o %s %s %s%s'%(opts, x, ' '.join(olist), self.COMPILER['libs'],lf)

DEFAULT_COMPILER = CompilerLocal

if __name__=='__main__':
    o = Options()
    o.process_args()  
##    make_bc(o.bc_spec())
##    gcc = Compiler(o.debug)
##    m = MakeMLMC(o, gcc)
    o.make.build()
    o.make.run()

"""
variables:

specified either in config file or on command line, or in answer to query

these are saved in config file separately from vars with default values
only these are used when config file read.

default values based on those in mlmc_main
and overridden by values in the chosen options

values used are those default values overridden by the specified values.

ie the default values are not calculated until after all the options are
determined.  constants likewise, but they don't have specified values.
"""

"""
Options for running:

   any time an option is set, default values for config vars for the option
   are set to option defaults

   options set to defaults
   
   If -r or -c specified, values read from file.

   any option:value settings on command line override defaults
   any config=value settings on command line override defaults

   if (no command line settings and not '-r' and not '-c') or '-a' then
      query user for all options

   if '-q' the run runs in quiet mode

examples:

make alsvid-uq -r                       runs same setup as last time
make alsvid-uq -r space:o2eno           runs same as last time, but in second order
make alsvid-uq -c test                  runs a standard test setup
make alsvid-uq -a                       queries user for all settings
"""
