
/*
 * GENERAL FVM routines
 */

#include "equation.h"
#include "multi.h"

real dt, t, t_halt, halt_times[NSAVES+SAVE0], dx[NDIR];
volume<ConservedVars, 1> U;
volume<ConservedVars, 1> Flux[NDIR];
volume<ConservedVars, 1> Source;
volume<XtraVars, 1> Xtra;
int multix, multiy, multiz, multi_main;

void step_finalize()
{
  for (int i = 0; i < NXl; i++) {
    for (int j = 0; j < NYl; j++) {
      for (int k = 0; k < NZl; k++) {
        calc_xtra(Xtra[i][j][k], U[i][j][k], i, j, k);
      }
    }
  }
  
  multi_bc();

  #if NX != 1
  set_ghost_x();
  #endif
  
  #if NY != 1
  set_ghost_y();
  #endif
  
  #if NZ != 1
  set_ghost_z();
  #endif
}

void model_init()
{
  // mesh parameters
  dx[X] = 1.0*MAXX/(MULTIXl*NXl);
  dx[Y] = 1.0*MAXY/(MULTIYl*NYl);
  dx[Z] = 1.0*MAXZ/(MULTIZl*NZl);
  
  // bottom topography
  #if TOPOGRAPHY
  for (int i = -GX; i < NXl+GX; i++)
    for (int j = -GY; j < NYl+GY; j++)
      Xtra[i][j][0].b = get_b(i,j);
  #endif
  
  // initial data
  PrimitiveVars pv;
  for (int i = 0; i < NXl; i++) {
    for (int j = 0; j < NYl; j++) {
      for (int k = 0; k < NZl; k++) {
        zero(pv);
        init_cell ( pv, (i+0.5+NXl*multix)*dx[X], (j+0.5+NYl*multiy)*dx[Y], (k+0.5+NZl*multiz)*dx[Z] );
        #if TOPOGRAPHY
        pv.h -= Xtra[i][j][0].b;
        if (pv.h < 0.0) pv.h = 0.0;
        #endif
        pv.toConservedVars(U[i][j][k]);
      }
    }
  }
}

#if ENTROPIC_RECONSTRUCTION

void ConservedVars_to_ReconstVars () {
  for (int i = -GX; i < NXl+GX; i++)
    for (int j = -GY; j < NYl+GY; j++) 
      ConservedVars_to_ReconstVars (U[i][j][0], Xtra[i][j][0]);
}

void ReconstVars_to_ConservedVars () {
  for (int i = -GX; i < NXl+GX; i++)
    for (int j = -GY; j < NYl+GY; j++) 
      ReconstVars_to_ConservedVars (U[i][j][0], Xtra[i][j][0]);
}

#endif
