
#include "equation.h"
#include "hll_util.h"

void hll_speeds(const AllVariables &lV, const AllVariables &rV, int d, real &sl, real &sr, real &cs)
{
  PrimitiveVars vh;
  real wl = sqrt(lV.U.rho);
  real wr = sqrt(rV.U.rho);

  vh.rho = (lV.U.rho + rV.U.rho)/2;
  vh.u = (lV.XU.u * wl + rV.XU.u * wr)/(wl+wr);
  vh.B = (lV.U.B * wl + rV.U.B * wr)/(wl+wr);
  vh.p = (lV.XU.p * wl + rV.XU.p * wr)/(wl+wr);
  real a2h = GAMMA * vh.p / vh.rho;
  real A = (a2h + vh.B.sqr()/vh.rho)/2;
  cs = ssqrt(A + ssqrt(A*A - a2h*vh.B[d]*vh.B[d]/vh.rho));
  sl = min(lV.XU.u[d] - lV.XU.cf[d], vh.u[d] - cs);
  sr = max(rV.XU.u[d] + rV.XU.cf[d], vh.u[d] + cs);
}

void hll_speeds_ext(const AllVariables &lV, const AllVariables &rV, int d, real eps, real &sl, real &sr, real &cs)
{

  PrimitiveVars vh;
  real wl = sqrt(lV.U.rho);
  real wr = sqrt(rV.U.rho);

  vh.rho = (lV.U.rho + rV.U.rho)/2;
  vh.u = (lV.XU.u * wl + rV.XU.u * wr)/(wl+wr);
  vh.B = (lV.U.B * wl + rV.U.B * wr)/(wl+wr);
  vh.p = (lV.XU.p * wl + rV.XU.p * wr)/(wl+wr);
  real a2h = GAMMA * vh.p / vh.rho;
  real A = (a2h + vh.B.sqr()/vh.rho)/2;
  cs = ssqrt(A + ssqrt(A*A - a2h*vh.B[d]*vh.B[d]/vh.rho));

  // calculate extended fast speed cf at the left boundary

  vector Bext = lV.U.B;
  Bext[d] = Bext[d]*sqrt(1+eps);
  real a2 = GAMMA*lV.XU.p/lV.U.rho;
  real b2 = Bext.sqr()/lV.U.rho;
  real cm2 = 0.5*(a2 + b2);
  real cfl = sqrt( cm2 + ssqrt(cm2*cm2 - a2*Bext[d]*Bext[d]/lV.U.rho ) );

  // calculate extended fast speed cf at the right boundary

  Bext = rV.U.B;
  Bext[d] = Bext[d]*sqrt(1+eps);
  a2 = GAMMA*rV.XU.p/rV.U.rho;
  b2 = Bext.sqr()/rV.U.rho;
  cm2 = 0.5*(a2 + b2);
  real cfr = sqrt( cm2 + ssqrt(cm2*cm2 - a2*Bext[d]*Bext[d]/rV.U.rho ) );
  
  real correct = 0.5*max(0,lV.XU.u[d]-rV.XU.u[d]);

  sl = min(lV.XU.u[d] + correct - cfl, vh.u[d] - cs);
  sr = max(rV.XU.u[d] - correct + cfr, vh.u[d] + cs);
}


