
#ifndef MAIN_H
#define MAIN_H

#include <math.h>

#include "config.h"
#include "equation.h"

// factor for number of samples in each succesive level
const int FACTOR = 1 << ORDER;

// total number of samples (from all levels)
const long_int M_TOTAL = ML * ( 1 - ( 1 << (ORDER * (L+1)) ) ) / ( 1 - FACTOR );

// resolution level at the coarsest mesh
const int L0 = (int) floor ( log2(NX) ) - L;

// === global variables

// level dependent quantities
extern long_int M[L+1];                // number of samples for each level
extern int N[NDIR][L+1];               // number of cells for each level

extern int MULTI[NDIR][L+1];           // number node groups working on the same domain
extern int NSAMPLERS[L+1];             // number node groups working on the same level samples
extern int NSAMPLERSl;                 // number node groups working on the same level samples
extern int ACTIVE_SAMPLERSl;           // number node groups working on the same downsampled level samples
extern int level_begin, level_end;     // levels to start and to end (might be different from 0 and L for multi:mpi)
extern long_int partition_size;        // numbe of samples per one sampler

// main containers for mean and variance
extern volume <ConservedVars, 1> mean_U [NSAVES+SAVE0];
extern volume <XtraVars, 1>      mean_X [NSAVES+SAVE0];
#if COMPUTE_VARIANCE
extern volume <ConservedVars, 1> var_U  [NSAVES+SAVE0];
extern volume <XtraVars, 1>      var_X  [NSAVES+SAVE0];
#endif

// level-dependent containers for mean and variance
extern volume <ConservedVars, 1> mean_U_l [NSAVES+SAVE0];
extern volume <XtraVars, 1>      mean_X_l [NSAVES+SAVE0];
#if COMPUTE_VARIANCE
extern volume <ConservedVars, 1> var_U_l  [NSAVES+SAVE0];
extern volume <XtraVars, 1>      var_X_l  [NSAVES+SAVE0];
#endif

// temporary volumes for computational efficiency
#if COMPUTE_VARIANCE
extern volume <ConservedVars, 1> temp_U_l;
extern volume <XtraVars,      1> temp_X_l;
#endif

// === timers
void start_timer();
void stop_timer();

#endif
