
#include "main.h"
#include "chaos.h"

///::title:: Wave equation with stochastic log-normal coefficient (1D version)
///::vars:: NX=512 MAXT=2.0 KL=10 KL_SCALING=0.1 KL_DECAY=2.5
///::consts:: MAXX=2 N_UNIFORM=0 N_NORMAL=KL
///::bc:: REPEAT

// u_0     = sin(pi*x);
// (u_0)_t = 0.0;

void init_cell(PrimitiveVars &v, real x, real y, real z)
{
  v.u    = sin(pi*x);
  v.v[0] = pi*cos(pi*x);
  v.w    = 0.0;
  
  // Karhunen–Loeve expantion of log(k) consisting of KL basis functions
  v.k    = 1.0;
  for (int m=1; m<=KL; m++)
    v.k *= exp(pow(m,-KL_DECAY) * sin(m*pi*x) * chaos.normal[m-1]);
  v.k *= KL_SCALING;
  
  v.rho  = 1.0;
}
