
#include <iostream>

#include "main.h"
#include "space.h"
#include "eno.h"

///::title:: Second Order in Space, positive fixed WENO
///::depends:: time_o2 substep_o12 powell_o23 eno
///::consts:: GHOST_SIZE=2 INDICATOR=1
///::vars:: ORDER=2 CFL=0.475

static const real EPSILON = 1e-8;
static const real BOUND = 1.9;

void left(AllVariables &V, int i, int j, int k, int d)
{
  real i0 = indicator(i, j, k, d);
  real b0 = sqr(indicator(i+DX[d], j+DY[d], k+DZ[d], d) - i0);
  real b1 = sqr(i0 - indicator(i-DX[d], j-DY[d], k-DZ[d], d));
  real a0 = 1/(3*(EPSILON + b0)*(EPSILON + b0));
  real a1 = 2/(3*(EPSILON + b1)*(EPSILON + b1));
  real w0 = a0 / (a0 + a1);
  real w1 = a1 / (a0 + a1);
  PrimitiveVars Pijk(U[i][j][k], Xtra[i][j][k]);
  PrimitiveVars dP = w1 * (Pijk - PrimitiveVars(U[i-DX[d]][j-DY[d]][k-DZ[d]], Xtra[i-DX[d]][j-DY[d]][k-DZ[d]])) +
    w0 * (PrimitiveVars(U[i+DX[d]][j+DY[d]][k+DZ[d]], Xtra[i+DX[d]][j+DY[d]][k+DZ[d]]) - Pijk);
  dP.rho = max(-BOUND*Pijk.rho, min(BOUND*Pijk.rho, dP.rho));
  dP.p = max(-BOUND*Pijk.p, min(BOUND*Pijk.p, dP.p));

  real LL = 0.125*Pijk.rho*(dP.u.sqr()) -
    0.5*min(0, dP.rho * (Pijk.u * dP.u)) +
    0.5*dP.rho*dP.rho*dP.u.sqr()/Pijk.rho;
  
  #if !NO_B
  LL += 0.125*dP.B.sqr();
  #endif
  
  if (Pijk.p < 0) {
    std::cerr<<"Negative Pressure left"<<i<<j<<k<<"\n";
    throw 1;
  }
  real R = Pijk.p / (GAMMA-1);
  real aijk = 0.5*sqrt(R/max(R,LL));
  
  calc_all_vars(V, Pijk-dP*aijk);
}

void right(AllVariables &V, int i, int j, int k, int d)
{
  real i0 = indicator(i, j, k, d);
  real b0 = sqr(indicator(i+DX[d], j+DY[d], k+DZ[d], d) - i0);
  real b1 = sqr(i0 - indicator(i-DX[d], j-DY[d], k-DZ[d], d));
  real a0 = 2/(3*(EPSILON + b0)*(EPSILON + b0));
  real a1 = 1/(3*(EPSILON + b1)*(EPSILON + b1));
  real w0 = a0 / (a0 + a1);
  real w1 = a1 / (a0 + a1);
  PrimitiveVars Pijk(U[i][j][k], Xtra[i][j][k]);
  PrimitiveVars dP = w1 * (Pijk - PrimitiveVars(U[i-DX[d]][j-DY[d]][k-DZ[d]], Xtra[i-DX[d]][j-DY[d]][k-DZ[d]])) +
    w0 * (PrimitiveVars(U[i+DX[d]][j+DY[d]][k+DZ[d]], Xtra[i+DX[d]][j+DY[d]][k+DZ[d]]) - Pijk);
  dP.rho = max(-BOUND*Pijk.rho, min(BOUND*Pijk.rho, dP.rho));
  dP.p = max(-BOUND*Pijk.p, min(BOUND*Pijk.p, dP.p));

  real LL = 0.125*Pijk.rho*(dP.u.sqr()) -
    0.5*min(0, dP.rho * (Pijk.u * dP.u)) +
    0.5*dP.rho*dP.rho*dP.u.sqr()/Pijk.rho;
  
  #if !NO_B
  LL += 0.125*dP.B.sqr();
  #endif
  
  if (Pijk.p < 0) {
    std::cerr<<"Negative Pressure right"<<i<<j<<k<<"\n";
    throw 1;
  }
  real R = Pijk.p / (GAMMA-1);
  real aijk = 0.5*sqrt(R/max(R,LL));
  
  calc_all_vars(V, Pijk+dP*aijk);
}
