
#include "main.h"
#include "space.h"
#include "eno.h"

///::title:: Third Order in Space, plain WENO
///::depends:: time_o3 substep_o3 powell_o23 eno
///::consts:: GHOST_SIZE=2 NQUAD=1 INDICATOR=1
///::vars:: ORDER=3 CFL=0.475

static const real EPSILON = 1e-8;

void left(AllVariables &V, int i, int j, int k, int d)
{
  real i0 = indicator(i, j, k, d);
  real b0 = sqr(indicator(i+DX[d], j+DY[d], k+DZ[d], d) - i0);
  real b1 = sqr(i0 - indicator(i-DX[d], j-DY[d], k-DZ[d], d));
  real a0 = 1/(3*(EPSILON + b0)*(EPSILON + b0));
  real a1 = 2/(3*(EPSILON + b1)*(EPSILON + b1));
  real w0 = a0 / (a0 + a1);
  real w1 = a1 / (a0 + a1);
  
  V.U = 0.5*(w1 * U[i-DX[d]][j-DY[d]][k-DZ[d]] + (3*w0+w1) * U[i][j][k] - w0 * U[i+DX[d]][j+DY[d]][k+DZ[d]]);
  
  calc_xtra(V.XU, V.U, i, j, k);
}

void right(AllVariables &V, int i, int j, int k, int d)
{
  real i0 = indicator(i, j, k, d);
  real b0 = sqr(indicator(i+DX[d], j+DY[d], k+DZ[d], d) - i0);
  real b1 = sqr(i0 - indicator(i-DX[d], j-DY[d], k-DZ[d], d));
  real a0 = 2/(3*(EPSILON + b0)*(EPSILON + b0));
  real a1 = 1/(3*(EPSILON + b1)*(EPSILON + b1));
  real w0 = a0 / (a0 + a1);
  real w1 = a1 / (a0 + a1);
  
  V.U = 0.5*(w0 * U[i+DX[d]][j+DY[d]][k+DZ[d]] + (3*w1+w0) * U[i][j][k] - w1 * U[i-DX[d]][j-DY[d]][k-DZ[d]]);

  calc_xtra(V.XU, V.U, i, j, k);
}
