
/* Wave (Wave equation)
 * Structures and operator overloading for ConservedVars, PrimitiveVars and XtraVars
 */

#ifndef STRUCTURES_WAVE_H
#define STRUCTURES_WAVE_H

#include "numtypes.h"

// === ConservedVars

struct ConservedVars {
  real   u;
  vector v;     // v = (u_x, u_y, y_z)
  real   w;     // w = u_t
  real   k;
  real   rho;
  
  real norm() const
  {
    return v.norm() + fabs(w) + fabs(rho) + fabs(k);
  }
  
  // defined in "structures.cpp"
  
  void operator += (const ConservedVars &a);
  void operator -= (const ConservedVars &a);
  void operator *= (const ConservedVars &a);
  void operator /= (const ConservedVars &a);
  void operator /= (const real r);
  void operator *= (const real r);
  void operator =  (const real r);
  #if COMPUTE_KURTOSIS
  void operator -= (const real r);
  #endif
};

// === XtraVars

struct XtraVars {
  real   E;
  
  // defined in "structures.cpp"
  
  void operator += (const XtraVars &a);
  void operator -= (const XtraVars &a);
  void operator *= (const XtraVars &a);
  void operator /= (const XtraVars &a);
  void operator /= (const real r);
  void operator *= (const real r);
  void operator =  (const real r);
  #if COMPUTE_KURTOSIS
  void operator -= (const real r);
  #endif
};

// === PrimitiveVars

struct PrimitiveVars {
  real   u;
  vector v;     // v = (u_x, u_y, y_z)
  real   w;     // w = u_t
  real   k;
  real   rho;
  
  void toConservedVars(ConservedVars &CV) const {
    CV.u   = u;
    CV.v   = v;
    CV.w   = w;
    CV.k   = k;
    CV.rho = rho;
  }
  
  PrimitiveVars() {}
  PrimitiveVars(const ConservedVars &CV, const XtraVars &XV);
};

// picks PrimitiveVars from ConservedVars (U) and XtraVars (Xtra)
inline PrimitiveVars::PrimitiveVars(const ConservedVars &CV, const XtraVars &XV)
{
  u   = CV.u;
  v   = CV.v;
  w   = CV.w;
  k   = CV.k;
  rho = CV.rho;
}

#endif
