
#include "main.h"
#include "multi.h"

///::title:: Third order Runge-Kutta timestepping

//....

static volume<ConservedVars, 0> save;

void time_step()
{
  if (multi_main && isnan(t)) {
    std::cout << " :: ERROR: time is NaN." << std::endl;
    throw 1;
  }
  
  save.reinit(NXl, NYl, NZl);
  
  for (int i = 0; i < NXl; i++) {
    for (int j = 0; j < NYl; j++) {
      for (int k = 0; k < NZl; k++) {
        save[i][j][k] = U[i][j][k];
      }
    }
  }
  
  sub_step(true);

  step_finalize();

  sub_step(false);

  for (int i = 0; i < NXl; i++) {
    for (int j = 0; j < NYl; j++) {
      for (int k = 0; k < NZl; k++) {
        U[i][j][k] = 0.25 * (U[i][j][k] + 3 * save[i][j][k]);
      }
    }
  }

  step_finalize();

  sub_step(false);

  for (int i = 0; i < NXl; i++) {
    for (int j = 0; j < NYl; j++) {
      for (int k = 0; k < NZl; k++) {
        U[i][j][k] = (2*U[i][j][k] + save[i][j][k])/3;
      }
    }
  }
  
  #ifndef QUIET
  if (PRINT_t && multi_main) std::cout << " :: time:  t=" << t << " (dt=" << dt << ")" << std::endl;
  #endif
  
  t += dt;
}
