
#include "topography.h"
#include "ec_util.h"

#include "equation.h"
#include "multi.h"

real get_b (const int &i, const int &j) {
  
  #if INTERPOLATE_TOPOGRAPHY
  if (HAVE_DIR(Y))
    return 0.5 * ( b_cell_interface (i, j) + b_cell_interface (i+1, j) ) * 0.5 * ( b_cell_interface (i, j) + b_cell_interface (i, j+1) );
  else
    return 0.5 * ( b_cell_interface (i, j) + b_cell_interface (i+1, j) );
  
  #else
  return b_cell_center (i, j);
  
  #endif
}

real b_cell_center (const int &i, const int &j) {
  if (HAVE_DIR(Y))
    return b ( (i + 0.5 + NXl*multix) * dx[X], (j + 0.5 + NYl*multiy) * dx[Y] );
  else {
    return b ( (i + 0.5 + NXl*multix) * dx[X], 1.0 );
  }
}

real b_cell_interface (const int &i, const int &j) {
  if (HAVE_DIR(Y))
    return b ( (i + NXl*multix) * dx[X], (j + NYl*multiy) * dx[Y] );
  else
    return b ( (i + NXl*multix) * dx[X], 1.0 );
}

#if !EC_TOPOGRAPHY_QUAD

// 2nd order accurate source quadrature
void compute_source (const int &i, const int &j) {
  Source[i][j][0].hu[X] = - GRAVITY * U[i][j][0].h * ( Xtra[i+1][j][0].b - Xtra[i-1][j][0].b ) / (2*dx[X]);
  if (HAVE_DIR(Y))
    Source[i][j][0].hu[Y] = - GRAVITY * U[i][j][0].h * ( Xtra[i][j+1][0].b - Xtra[i][j-1][0].b ) / (2*dx[Y]);
}

#endif
