
#include "topography.h"

#include "equation.h"
#include "multi.h"
#include "chaos.h"

#include "topography-hierarchical.h"

///::title:: Hierarchical hat basis representation (1D version)
///::vars:: HR=9 ANTI_ALIASING=1
///::consts:: HR_DECAY=1.5
///::consts:: T_UNIFORM=(1<<HR)-1 T_NORMAL=0
///::depends:: topography-hierarchical
///::macro:: LEVEL_INIT_HOOK topography-hierarchical.h hierarchical_init(NXl*MULTIXl, NYl*MULTIYl);

real b (real x, real y) {
  
  real value = 0.0;
  long_int counter = 0;
  
  // mean for bottom topography
  if (HR_MAX >= 3) value += 1.0 * hr_basis_function(x/MAXX,3,1);
  if (HR_MAX >= 4) value -= 0.8 * hr_basis_function(x/MAXX,4,4);
  if (HR_MAX >= 5) value += 0.3 * hr_basis_function(x/MAXX,5,10);
  
  // uncertainty perturbations
  for (int m=1; m<=HR_MAX; m++)
    for (int j=0; j<J[m]; j++)
      value += C[m] * hr_basis_function(x/MAXX,m,j) * (2*chaos.t_uniform[counter++] - 1);
  
  return BED_OFFSET + BED_SCALING * value;
}
