
#include "topography.h"

#include "equation.h"
#include "multi.h"
#include "chaos.h"

#include "topography-hierarchical.h"

///::title:: Hierarchical hat basis representation (2D version, isotropic tensorization)
///::vars:: HR=7 ANTI_ALIASING=1
///::consts:: HR_DECAY=1.0
///::consts:: T_UNIFORM=((1<<HR)-1)*((1<<HR)-1) T_NORMAL=0
///::depends:: topography-hierarchical
///::macro:: LEVEL_INIT_HOOK topography-hierarchical.h hierarchical_init(NXl*MULTIXl, NYl*MULTIYl);

real b (real x, real y) {
  
  real value = 0.0;
  long_int counter = 0;
  
  // mean for bottom topography
  if (HR_MAX >= 3) value += 1.0 * hr_basis_function(x/MAXX,3,1)  * hr_basis_function(y/MAXY,3,1);
  if (HR_MAX >= 4) value -= 0.8 * hr_basis_function(x/MAXX,4,4)  * hr_basis_function(y/MAXY,4,4);
  if (HR_MAX >= 5) value += 0.3 * hr_basis_function(x/MAXX,5,10) * hr_basis_function(y/MAXY,5,10);
  
  // uncertainty perturbations (implementation for REPRODUCIBLE results)
  for (int m=1; m<=HR_MAX; m++) {
    
    // isotropic terms of standard product of basis functions
    for (int jx=0; jx<J[m]; jx++)
      for (int jy=0; jy<J[m]; jy++)
        value += C[m] * hr_basis_function(x/MAXX,m,jx) * hr_basis_function(y/MAXY,m,jy) * (2*chaos.t_uniform[counter++] - 1);
    
    // additional isotropic terms (shifted in x direction)
    for (int jx=0; jx<J[m]-1; jx++)
      for (int jy=0; jy<J[m]; jy++)
        value += C[m] * hr_shifted_basis_function(x/MAXX,m,jx) * hr_basis_function(y/MAXY,m,jy) * (2*chaos.t_uniform[counter++] - 1);
    
    // additional isotropic terms (shifted in y direction)
    for (int jx=0; jx<J[m]; jx++)
      for (int jy=0; jy<J[m]-1; jy++)
        value += C[m] * hr_basis_function(x/MAXX,m,jx) * hr_shifted_basis_function(y/MAXY,m,jy) * (2*chaos.t_uniform[counter++] - 1);
  }
  
  return BED_OFFSET + BED_SCALING * value;
}
