
#include "topography.h"

#include "equation.h"
#include "multi.h"
#include "chaos.h"

#include "topography-wavelet.h"

///::title:: Haar wavelet representation (1D version)
///::vars:: WL=9
///::consts:: WL_DECAY=1.1 WL_MIN_CORR_LEVEL=2
///::consts:: T_UNIFORM=(1<<WL) T_NORMAL=0 INTERPOLATE_TOPOGRAPHY=1
///::depends:: topography-wavelet
///::macro:: LEVEL_INIT_HOOK topography-wavelet.h wavelet_init(NXl*MULTIXl, NYl*MULTIYl);

real b (real x, real y) {
  
  real value = 0.0;
  long_int counter = 0;
  
  value += 0.1 * (2*chaos.t_uniform[0] - 1);
  counter++;
  
  for (int m=max(1,WL_MIN_CORR_LEVEL); m<=WL; m++) {
    for (int j=0; j<J[m-1]; j++)
      value += C[m] * wavelet(x/MAXX,m,j) * (2*chaos.t_uniform[counter++] - 1);
  }
  
  return BED_OFFSET +  + BED_SCALING * 0.5 * value;
}
