
import numpy

class Variable (object):
    def __init__(self, i, name):
        self.i = i
        self.name = name
    
    def load(self, r, ts, name):
        d = r.load(self.i, ts)
        return mlab.pipeline.scalar_field(d, name=name)

class VectorVariable (object):
    def __init__(self, i, name):
        self.i = i
        self.name = name

    def load(self, r, ts, name):
        x = r.load(self.i, ts)
        y = r.load(self.i+1, ts)
        z = r.load(self.i+2, ts)
        return mlab.pipeline.vector_field(x, y, z, name=name)

class Config (object):
    def __init__(self, run, conv):
        self.run = run
        self.conv = conv

    def __getattr__(self, n):
        return self.conv(self.run.vars[n])

# MHD
class MHD (object):
    RHO = Variable(0, 'rho')
    
    M  = VectorVariable(1, 'm')
    MX = Variable      (1, 'mx')
    MY = Variable      (2, 'my')
    MZ = Variable      (3, 'mz')
    
    E = Variable(4, 'E')
    
    B  = VectorVariable(5, 'B')
    BX = Variable      (5, 'Bx')
    BY = Variable      (6, 'By')
    BZ = Variable      (7, 'Bz')
    
    P = Variable(8, 'p')
    
    U  = VectorVariable(9,  'u')
    UX = Variable      (9,  'ux')
    UY = Variable      (10, 'uy')
    UZ = Variable      (11, 'uz')
    
    A = Variable(12, 'a')
    
    CF  = VectorVariable(13, 'cf')
    CFX = Variable      (13, 'cfx')
    CFY = Variable      (14, 'cfy')
    CFZ = Variable      (15, 'cfz')
    
    NVARS=16

mhd = MHD()

# Burgers'
class Burgers (object):
    U = Variable(0, 'u')
    E = Variable(1, 'E')
    NVARS = 2

br = Burgers()

# Euler
class Euler (object):
    RHO = Variable(0, 'rho')
    
    M  = VectorVariable(1, 'm')
    MX = Variable      (1, 'mx')
    MY = Variable      (2, 'my')
    MZ = Variable      (3, 'mz')
    
    E = Variable(4, 'E')
    P = Variable(5, 'p')
    
    U  = VectorVariable(6,  'u')
    UX = Variable      (6, 'ux')
    UY = Variable      (7, 'uy')
    UZ = Variable      (8, 'uz')
    
    NVARS = 9

eu = Euler()

# Shallow Water
class ShallowWater (object):
    H = Variable(0, 'h+b')
    
    HU  = VectorVariable(1, 'hu')
    HUX = Variable      (1, 'hux')
    HUY = Variable      (2, 'huy')
    #HUZ should not exist
    
    U  = VectorVariable(4,  'u')
    UX = Variable      (4, 'ux')
    UY = Variable      (5, 'uy')
    #UZ should not exist
    
    E = Variable(7, 'E')
    B = Variable(8, 'b')
    
    NVARS = 9

sw = ShallowWater()

# Wave
class Wave (object):
    U  = Variable(0, 'u')
    
    V  = VectorVariable(1, 'v')
    VX = Variable      (1, 'vx')
    VY = Variable      (2, 'vy')
    VZ = Variable      (3, 'vz')
    
    W = Variable(4, 'w')
    K = Variable(5, 'a')
    RHO = Variable(6, 'rho')
    E = Variable(7, 'E')
    
    NVARS = 8

wa = Wave()

def get_NVARS (equation):
    if equation == 'burgers':
      return br.NVARS
    elif equation == 'sw':
      return sw.NVARS
    elif equation == 'wave':
      return wa.NVARS
    elif equation == 'euler':
      return eu.NVARS
    elif equation == 'mhd':
      return mhd.NVARS